"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require("react");

var _propTypes = require("prop-types");

var _propTypes2 = _interopRequireDefault(_propTypes);

var _axios = require("axios");

var _axios2 = _interopRequireDefault(_axios);

var _lodash = require("lodash.debounce");

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require("lodash.isequal");

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require("lodash.isplainobject");

var _lodash6 = _interopRequireDefault(_lodash5);

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultFetchCreator = function defaultFetchCreator(requestData) {
  return (0, _axios2.default)(requestData);
};

var AsyncFetcher = function (_PureComponent) {
  (0, _inherits3.default)(AsyncFetcher, _PureComponent);

  function AsyncFetcher(props) {
    (0, _classCallCheck3.default)(this, AsyncFetcher);

    var _this = (0, _possibleConstructorReturn3.default)(this, _PureComponent.call(this, props));

    _this.setCustomState = function (state) {
      if (!!state && ((typeof state === "undefined" ? "undefined" : (0, _typeof3.default)(state)) === "object" || typeof state === "function")) {
        var hasCustom = false;
        var customState = _this.state.customState;

        if (typeof state === "function") {
          state = state((0, _extends3.default)({
            params: _this.state.params,
            postData: _this.state.postData
          }, customState));
          if ((typeof state === "undefined" ? "undefined" : (0, _typeof3.default)(state)) !== "object") {
            return;
          }
        }

        (0, _keys2.default)(state).forEach(function (k) {
          if (["params", "postData"].indexOf(k) !== -1) {
            if (!(0, _lodash4.default)(_this.state[k], state[k])) {
              var _this$setState;

              _this.setState((_this$setState = {}, _this$setState[k] = state[k], _this$setState));
            }
          } else {
            customState[k] = state[k];
            hasCustom = true;
          }
        });

        if (hasCustom) {
          _this.setState({ customState: customState });
        }
      }
    };

    _this._mounted = false;


    _this.state = {
      isLoading: !!props.autoFetch,
      response: null,
      data: null,
      error: null,
      params: null,
      postData: null,
      customState: {}
    };
    return _this;
  }

  AsyncFetcher.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var _this2 = this;

    var hasChange = false;
    ["method", "params", "postData", "url", "authToken"].forEach(function (k) {
      if (k in prevProps && !(0, _lodash4.default)(prevProps[k], _this2.props[k])) {
        hasChange = true;
      }
    });
    if (!hasChange) {
      ["params", "postData"].forEach(function (k) {
        if (k in prevState && !(0, _lodash4.default)(prevState[k], _this2.state[k])) {
          hasChange = true;
        }
      });
    }

    if (+prevProps.debounce !== +this.props.debounce) {
      this._debouncedFetch = +this.props.debounce > 0 ? (0, _lodash2.default)(this.fetch.bind(this), +this.props.debounce) : this.fetch.bind(this);
    }

    if (hasChange) {
      this.refetch();
    }
  };

  AsyncFetcher.prototype.componentDidMount = function componentDidMount() {
    this._mounted = true;
    this._debouncedFetch = +this.props.debounce > 0 ? (0, _lodash2.default)(this.fetch.bind(this), +this.props.debounce) : this.fetch.bind(this);

    if (!!this.props.autoFetch) {
      this.fetch();
    }
  };

  AsyncFetcher.prototype.componentWillUnmount = function componentWillUnmount() {
    this._mounted = false;
  };

  AsyncFetcher.prototype.refetch = function refetch() {
    if (!!this.props.autoFetch) {
      this._debouncedFetch.apply(this, arguments);
    }
  };

  AsyncFetcher.prototype.fetch = function fetch() {
    var _this3 = this;

    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
    var customRequest = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var _props = this.props,
        url = _props.url,
        headers = _props.headers,
        axiosConfig = _props.axiosConfig,
        authToken = _props.authToken,
        ajax = _props.ajax,
        accepts = _props.accepts,
        responseType = _props.responseType,
        withCredentials = _props.withCredentials,
        transformError = _props.transformError,
        fetchCreator = _props.fetchCreator,
        onResponse = _props.onResponse,
        onError = _props.onError;


    var method = this.props.method.toLowerCase();
    var isPost = method === "post" || method === "put";
    var contentType = !!this.props.contentType ? (0, _utils.parseMime)(this.props.contentType) : isPost ? (0, _utils.parseMime)("form") : null;

    var customHeaders = {};
    if (!!contentType) {
      customHeaders["Content-Type"] = contentType;
    }
    if (!!authToken) {
      var authorization = typeof authToken === "function" ? authToken() : authToken;
      if (!!authorization) {
        if (authorization.indexOf("Bearer") === -1) {
          authorization = "Bearer " + authorization;
        }
        customHeaders["Authorization"] = authorization;
      }
    }
    if (!!ajax) {
      customHeaders["X-Requested-With"] = "XMLHttpRequest";
    }
    if (!!accepts) {
      customHeaders["Accept"] = function () {
        if (typeof accepts === "string") {
          return accepts.split(";").map(_utils.parseMime).join(";");
        }
        if (Array.isArray(accepts)) {
          return accepts.map(_utils.parseMime).join(";");
        }
        if (typeof accepts === "function") {
          return accepts();
        }
        return accepts;
      }();
    }

    var requestConfig = {
      url: url,
      method: method,
      withCredentials: !!withCredentials,
      headers: (0, _extends3.default)({}, headers, customHeaders)
    };
    if (isPost) {
      if (!data || (0, _utils.isEvent)(data)) {
        data = (0, _utils.concatParams)(this.props.postData, this.state.postData);
      }

      if (!!data && contentType.indexOf("x-www-form-urlencoded") !== -1 && (0, _lodash6.default)(data)) {
        data = (0, _utils.serialize)(data);
      }

      requestConfig.data = data;
    }

    var params = !isPost && !!data ? data : (0, _utils.concatParams)(this.props.params, this.state.params);
    if (!!params) {
      if (typeof params !== "string" && (0, _lodash6.default)(params)) {
        params = (0, _utils.serialize)(params);
      }
      if (typeof params === "string") {
        requestConfig.url = (requestConfig.url + (requestConfig.url.indexOf("?") === -1 ? "?" : "&") + params).replace(/\?&/gi, "?");
      } else {
        requestConfig.params = params;
      }
    }
    if (!!responseType) {
      requestConfig.responseType = responseType;
    }
    if (!!data) {
      requestConfig.data = data;
    }

    this.setState({ isLoading: true, error: null }, function () {
      return fetchCreator((0, _extends3.default)({}, requestConfig, axiosConfig, customRequest)).then(function (response) {
        if (!_this3._mounted) {
          return;
        }

        _this3.setState({
          response: response,
          data: response.data,
          isLoading: false
        }, function () {
          if (!!onResponse) onResponse(response);
        });
      }).catch(function (error) {
        if (!_this3._mounted) {
          return;
        }

        var transformedError = transformError((0, _extends3.default)({}, error));

        _this3.setState(function () {
          var errorState = {
            error: "" + (!!transformedError && typeof transformedError === "string" ? transformedError : error),
            response: null,
            data: null,
            isLoading: false
          };
          if (!!transformedError && (typeof transformedError === "undefined" ? "undefined" : (0, _typeof3.default)(transformedError)) === "object" && "response" in transformedError) {
            errorState.response = transformedError.response;

            if (!!errorState.response && (0, _typeof3.default)(errorState.response) === "object" && "data" in errorState.response) {
              errorState.data = errorState.response.data;
            }
          }

          return errorState;
        }, function () {
          if (!!onError) onError(transformedError);
        });
      });
    });
  };

  AsyncFetcher.prototype.render = function render() {
    if (!!this.props.children && typeof this.props.children === "function") {
      return this.props.children({
        fetch: this._debouncedFetch,
        isLoading: this.state.isLoading,
        response: this.state.response,
        data: this.state.data,
        error: this.state.error,
        params: this.state.params,
        postData: this.state.postData,
        state: this.state.customState,
        set: this.setCustomState
      }) || null;
    }

    return null;
  };

  return AsyncFetcher;
}(_react.PureComponent);

AsyncFetcher.propTypes = {
  autoFetch: _propTypes2.default.bool,
  method: _propTypes2.default.oneOf(["get", "post", "put", "delete", "options"]),
  url: _propTypes2.default.string.isRequired,
  params: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.object]),
  postData: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.object]),
  headers: _propTypes2.default.object,
  axiosConfig: _propTypes2.default.object,
  ajax: _propTypes2.default.bool,
  authToken: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.func]),
  accepts: _propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.array]),
  contentType: _propTypes2.default.string,
  responseType: _propTypes2.default.string,
  withCredentials: _propTypes2.default.bool,
  transformError: _propTypes2.default.func,
  fetchCreator: _propTypes2.default.func,
  onResponse: _propTypes2.default.func,
  onError: _propTypes2.default.func,
  debounce: _propTypes2.default.number,
  children: _propTypes2.default.func
};
AsyncFetcher.defaultProps = {
  autoFetch: true,
  method: "get",
  headers: {},
  axiosConfig: {},
  ajax: false,
  withCredentials: false,
  transformError: function transformError(err) {
    return err;
  },
  fetchCreator: defaultFetchCreator,
  debounce: 0
};
exports.default = AsyncFetcher;